/* $Id: common.c,v 1.9 2010/07/11 00:01:03 lamonte Exp $
 *
 * common.c
 *
 * Additional acknowledgements, news, and updates to the Qodem Project
 * are available at http://qodem.sourceforge.net .
 *
 * This module is licensed under the GNU General Public License Version 2.
 * Please see the file "COPYING" in this directory for more information
 * about the GNU General Public License Version 2.
 *
 *     Copyright (C) 2003  Kevin Lamonte
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#include "common.h"
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <sys/poll.h>
#include "status.h"
#include "modem.h"

/*
 * Wcsdup
 */
wchar_t * Xwcsdup(const wchar_t * ptr, const char * file, const int line) {

	wchar_t * local_ptr = NULL;
#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
	int length;
	int i;
#endif

	assert(ptr != NULL);

#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
	length = wcslen(ptr) + 1;
#ifdef DEBUG
	local_ptr = (wchar_t *)GC_debug_malloc(length * sizeof(wchar_t), file, line);
#else
	local_ptr = (wchar_t *)GC_malloc(length * sizeof(wchar_t));
#endif /* DEBUG */

	if (local_ptr != NULL) {
		for (i = 0; i < length; i++) {
			local_ptr[i] = ptr[i];
		}
	}
	local_ptr[length - 1] = 0;
#else
#ifdef __linux
	local_ptr = wcsdup(ptr);
#else
	local_ptr = (wchar_t *)Xmalloc((sizeof(wchar_t) + 1) * wcslen(ptr), __FILE__, __LINE__);
	wcsncpy(local_ptr, ptr, wcslen(ptr));
	local_ptr[wcslen(ptr)] = 0;
#endif /* __linux */
#endif

#if 0
/* For debugging: */
fprintf(stderr, "Xwcsdup: ptr=%p local_ptr=%p file=%s line=%d\n", ptr, local_ptr, file, line);
#endif

	return local_ptr;
} /* ---------------------------------------------------------------------- */

wchar_t * Xstring_to_wcsdup(const char * ptr, const char * file, const int line) {

	wchar_t * local_ptr = NULL;
	int length;

#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
	int i;
#endif

	assert(ptr != NULL);

	length = mbstowcs(NULL, ptr, 0) + 1;

#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
#ifdef DEBUG
	local_ptr = (wchar_t *)GC_debug_malloc((sizeof (wchar_t)) * length, file, line);
#else
	local_ptr = (wchar_t *)GC_malloc((sizeof (wchar_t)) * length);
#endif /* DEBUG */
#else
	local_ptr = (wchar_t *)malloc((sizeof (wchar_t)) * length);
#endif

	if (length > 1) {
		mbstowcs(local_ptr, ptr, length - 1);
		local_ptr[length - 1] = 0;
	} else {
		local_ptr[0] = 0;
	}

#if 0
/* For debugging: */
fprintf(stderr, "Xstring_to_wcsdup: ptr=%p local_ptr=%p file=%s line=%d\n", ptr, local_ptr, file, line);
#endif

	return local_ptr;
} /* ---------------------------------------------------------------------- */

/*
 * Strdup
 */
char * Xstrdup(const char * ptr, const char * file, const int line) {

	char * local_ptr = NULL;
#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
	int length;
	int i;
#endif

	assert(ptr != NULL);

#if defined(QODEM_USE_GC) && defined(HAVE_LIBGC)
	length = strlen(ptr) + 1;
#ifdef DEBUG
	local_ptr = (char *)GC_debug_malloc(length, file, line);
#else
	local_ptr = (char *)GC_malloc(length);
#endif /* DEBUG */

	if (local_ptr != NULL) {
		for (i=0;i<length;i++) {
			local_ptr[i] = ptr[i];
		}
	}
#else
	local_ptr = strdup(ptr);
#endif

#if 0
/* For debugging: */
fprintf(stderr, "Xstrdup: ptr=%p local_ptr=%p file=%s line=%d\n", ptr, local_ptr, file, line);
#endif

	return local_ptr;
} /* ---------------------------------------------------------------------- */

/* Function to clean out any characters waiting in stdin */
void purge_stdin() {
	struct pollfd pfd;

	pfd.fd = STDIN_FILENO;
	pfd.events = POLLIN;

	int rc = poll(&pfd, 1, 10);
	if (rc < 0) {
		/* Error */
	} else if (rc > 0) {

		/* Flush stdin */
		do {
			pfd.fd = STDIN_FILENO;
			pfd.events = POLLIN;
			getchar();
		} while ((rc = poll(&pfd, 1, 0)) > 0);
	}
} /* ---------------------------------------------------------------------- */
